namespace CSharpDynamicProxy {

	using System;
	using System.Reflection;
	using System.Runtime.Remoting.Proxies;
	using System.Runtime.Remoting.Messaging;

	public class LoggingProxy : RealProxy {
		private object proxiedObject;

		public static object GetProxy(object proxiedObject, Type theInterface) {
			LoggingProxy theProxy = new LoggingProxy(proxiedObject, theInterface);
			return theProxy.GetTransparentProxy();
		}

		private LoggingProxy(object proxiedObject, Type type) : base(type) {
			this.proxiedObject = proxiedObject;
		}
			
		public override IMessage Invoke(IMessage request) {
			IMethodCallMessage methodMessage = request as IMethodCallMessage;

			if (methodMessage != null) {
				MethodBase method = methodMessage.MethodBase;
			
				try {
					LogCall(method);
					object returnValue = method.Invoke(proxiedObject, methodMessage.Args);
					return new ReturnMessage(returnValue, null, 0, methodMessage.LogicalCallContext, methodMessage);
				} catch (Exception e) {
					return new ReturnMessage(e, methodMessage);
				}
			}
			return null;
		}

		private void LogCall(MethodBase method) {
			Console.WriteLine(method.ToString());
		}
	}
}