
namespace RDBMExample.Mapping {

	using System;
	using System.Data;
	using System.Data.SqlClient;
	using System.Collections;

	public delegate object FindObjectById(long oid);
	public delegate IList FindListById(long oid);

	public class EmployeeMapper {
		private const string FindByIDSql =  @"SELECT * FROM tbl_Employee WHERE ID=@ID";
		private const string FindMyManagerSql = @"SELECT * FROM tbl_Employee WHERE ManagerID=@ID";
		private Hashtable loadedObjects = new Hashtable();

		public object Find(long oid) {
			IList emps = Find(FindByIDSql, oid);
			if (emps.Count>0)
				return emps[0];
			return null;
		}
		public IList FindByManager(long oid) {
			return Find(FindMyManagerSql, oid);
		}
		
		private IList Find(string sql, long oid) {
			ArrayList retVal = new ArrayList();
			IDbConnection connection = GetNewConnection();
			connection.Open();
			try {
				IDataReader reader = GetReader(connection, sql, oid);
				while (reader.Read())
					retVal.Add( Load(reader) );
			} catch (Exception e) {
				throw new ApplicationException("Problem in Mapper " + this.GetType().ToString(), e);
			} finally {
				connection.Close();
			}
			return retVal;
		}

		private object Load(IDataReader reader) {
			long oid = reader.GetInt64(0);
			if (loadedObjects.Contains(oid)) return loadedObjects[oid];
			
			Employee emp = new Employee(oid);
			emp.FirstName = reader.GetString(1);
			emp.LastName = reader.GetString(2);
			if (!reader.IsDBNull(3))
				emp.Manager = (IEmployee) LazyLoadObjectProxy.GetProxy(new FindObjectById(Find), reader.GetInt64(3), typeof(IEmployee));
			emp.Manages = (IList) LazyLoadListProxy.GetProxy(new FindListById(FindByManager), oid);

			loadedObjects[oid] = emp;
			return emp;
		}

		private IDataReader GetReader(IDbConnection connection, string sql, long oid) {
			// Get a command
			IDbCommand command = connection.CreateCommand();
			command.CommandText = sql;
			// Add the ID parameter
			IDbDataParameter parm = command.CreateParameter();
			parm.ParameterName = "@ID";
			parm.Value = oid;
			command.Parameters.Add(parm);
			// Return the reader
			return command.ExecuteReader();
		}

		private IDbConnection GetNewConnection() {
			//return new SqlConnection(System.Configuration.ConfigurationSettings.AppSettings["ConnectionString"]);
			return new SqlConnection("Data Source=(local);Initial Catalog=Scheduling;user id=sa; password=;");
		}
	}
}
