namespace RDBMExample.Mapping {

	using System;
	using System.Collections;
	using System.Reflection;
	using System.Runtime.Remoting.Proxies;
	using System.Runtime.Remoting.Messaging;

	public class LazyLoadListProxy : RealProxy {
		private FindListById finder;
		private long oid;
		private IList proxiedList;

		public static object GetProxy(FindListById finder, long oid ) {
			LazyLoadListProxy theProxy = new LazyLoadListProxy(finder, oid);
			return theProxy.GetTransparentProxy();
		}

		private LazyLoadListProxy(FindListById finder, long oid) : base(typeof(IList)) {
			this.finder = finder;
			this.oid = oid;
			proxiedList = null;
		}
			
		public override IMessage Invoke(IMessage request) {
			//if we haven't loaded the list, load it now
			if (proxiedList == null)
				proxiedList = finder(oid);

			IMethodCallMessage methodMessage = request as IMethodCallMessage;

			if (methodMessage != null) {
				MethodBase method = methodMessage.MethodBase;
				try {
					object returnValue = method.Invoke(proxiedList, methodMessage.Args);
					return new ReturnMessage(returnValue, null, 0, methodMessage.LogicalCallContext, methodMessage);
				} catch (Exception e) {
					return new ReturnMessage(e, methodMessage);
				}
			}
			return null;
		}

	}
}