namespace RDBMExample.Mapping {


	using System;
	using System.Collections;
	using System.Reflection;
	using System.Runtime.Remoting.Proxies;
	using System.Runtime.Remoting.Messaging;

	public class LazyLoadObjectProxy : RealProxy {
		private FindObjectById finder;
		private long oid;
		private object proxiedObject;

		public static object GetProxy(FindObjectById finder, long oid, Type theInterface) {
			LazyLoadObjectProxy theProxy = new LazyLoadObjectProxy(finder, oid, theInterface);
			return theProxy.GetTransparentProxy();
		}

		private LazyLoadObjectProxy(FindObjectById finder, long oid, Type type) : base(type) {
			this.finder = finder;
			this.oid = oid;
			proxiedObject = null;
		}
			
		public override IMessage Invoke(IMessage request) {
			//if we haven't loaded the object, load it now
			if (proxiedObject == null)
				proxiedObject = finder(oid);

			IMethodCallMessage methodMessage = request as IMethodCallMessage;

			if (methodMessage != null) {
				MethodBase method = methodMessage.MethodBase;
				try {
					object returnValue = method.Invoke(proxiedObject, methodMessage.Args);
					return new ReturnMessage(returnValue, null, 0, methodMessage.LogicalCallContext, methodMessage);
				} catch (Exception e) {
					return new ReturnMessage(e, methodMessage);
				}
			}
			return null;
		}

	}
}