// The RootFinder.java file

public class RootFinder
{

  // Declaration of the Native (C) function
  private native int rootfinder(String funName,
                                double a, double b);

  // Member variables to hold results returned by rootfind
  double result;
  int niterations;

  static
    {
      // The runtime system executes a class's static
      // initializer when it loads the class.
      System.loadLibrary("CJavaInterface");
    }

  // An example function to find a root of: x*x - 2.
  private double myFunction(double x)
    {
      return x * x - 2.0;
    }

  // Another example function: x - cos(x).
  private double myFunction2(double x)
    {
      return x - Math.cos(x);
    }

  // The main program
  public static void main(String[] args)
    {
      double a, b;

      // Create an object of class RootFinder
      RootFinder rf = new RootFinder();

      System.out.println();
      System.out.println("Calls of library root finding routine rootfinder");
      System.out.println();

      // Search for a root of the first example function
      a = 1.0;
      b = 2.0;
      System.out.println("Root of x*x - 2");
      rf.FindRoot("myFunction", a, b);

      // Search for a root of the second example function
      a = 0.0;
      b = 5.0;
      System.out.println("Root of x - cos(x)");
      rf.FindRoot("myFunction2", a, b);

    }

  // A routine to find a root by calling the native (C) function
  private void FindRoot(String functionName, double a, double b)
    {
      int resCode;

      resCode = rootfinder(functionName, a, b);

      // Check the result code returned by the C function.
      if (resCode == -1)
        System.out.println("Cannot load library rootlib.dll / librootlib.so");
      else if (resCode == -2)
        System.out.println("Cannot find function rootfinder in rootlib.dll / librootlib.so");
      else if (resCode == -3)
        System.out.println("Cannot find method " + functionName +
                           " with signature (D)D");
      else if (resCode > 0)
        {
          System.out.print("rootlib function rootfinder returned non-zero exit code: ");
          System.out.println(resCode);
        }
      else
        {
          // resCode = 0 - we got some results.
          System.out.print("Lower bound of root = ");
          System.out.println(a);
          System.out.print("Upper bound of root = ");
          System.out.println(b);
          System.out.print("Computed root = ");
          System.out.println(result);
          System.out.print("Number of iterations = ");
          System.out.println(niterations);
        }
      System.out.println();
    }
}
