/* The RootFinderImp.c file */

#include <jni.h>         /* Java Native Interface headers */
#include "RootFinder.h"  /* Auto-generated header created by javah -jni */
#include <stdio.h>
#include <math.h>

/* Prototype for the rootlib function rootfinder */
#ifndef unix
/* This for VC++ on Windows to export the function rootfinder */
__declspec(dllexport)
#endif
double rootfinder(double (* f)(double x),
                  double a, double b,
                  int *nits, int *fail);

/* Global variables; they are global because they are
   required by the function evaluator rootfun, but come from
   Java via the native interface routine Java_RootFinder_rootfinder. */
JNIEnv *globalJavaEnv;
jobject globalJavaObject;
jmethodID globalMid;


/* This is the interface to the Java function which is to
   be searched for a root. */
double rootfun(double x)
{
  jdouble res;

  /* Here's where we call back to the user's function in the Java code */
  res = (*globalJavaEnv)->CallDoubleMethod(globalJavaEnv, globalJavaObject,
                                           globalMid, (jdouble)x);
  return (double)res;
}



/* Our C definition of the function rootfinder declared in RootFinder.java.
   The return value is an error code.
   Other results are returned via direct JNI access to Java object
   member variables. */
JNIEXPORT jint JNICALL
  Java_RootFinder_rootfinder(JNIEnv *env, jobject obj,
                             jstring funName,
                             jdouble a, jdouble b)
{
  int fail;
  double result;
  int nits;
  jclass cls;
  const char *functionName;
  jfieldID fid;
  int retVal;

  /* Copy the Java env pointers to global space
     so that rootfun can access them. */
  globalJavaEnv = env;
  globalJavaObject = obj;

  /* Get hold of the name of the user's Java evaluation function. */
  functionName = (*env)->GetStringUTFChars(env, funName, 0);

  /* Now we have the Java evaluation function name we
     can use it to get hold of a handle (method ID) to the function.
     Once more, the method ID is stored globally so that rootfun
     can use it. Note that the Java function signature must be
     "(D)D" (i.e. function with double argument, returning double). */
  cls = (*env)->GetObjectClass(env, obj);
  globalMid = (*env)->GetMethodID(env, cls, functionName, "(D)D");

  /* Free up the Java string argument so we don't leak memory. */
  (*env)->ReleaseStringUTFChars(env, funName, functionName);

  if (globalMid == 0)
    /* Cannot find method "functionName" with signature (D)D */
    return -1;
  else
    {
      /* Now call the function we're interested in from the rootlib C Library.
         rootfun is the function that we want to find a root of. */
      result = rootfinder(rootfun, (double)a, (double)b, &nits, &fail);

      if (fail == 0)
        {
          /* Put the results back to Java. */
          /* Get the ID of the Java RootFinder class member variable
             "result" (which is of type double, hence the "D" signature). */
          fid = (*env)->GetFieldID(env, cls, "result", "D");
          /* Set the result value via the ID */
          (*env)->SetDoubleField(env, obj, fid, result);
          /* In the same way, set the number of iterations. Int member
             variable "niterations" has signature "I". */
          fid = (*env)->GetFieldID(env, cls, "niterations", "I");
          (*env)->SetIntField(env, obj, fid, nits);
        }
    }

  /* Return any fail code that the rootlib.dll function rootfinder returned. */
  return fail;
}
