/* The file rootlib.c */

/* Searches function f(x) for a root in the interval [a,b],
   using a bisection method. */
#ifndef unix
/* This for VC++ on Windows to export the function rootfinder */
__declspec(dllexport)
#endif
double rootfinder(double (* f)(double x),
                  double a, double b,
                  int *nits, int *fail)
{
  double y, ya, yb, midpoint, ym;
  int n = 0;

  ya = f(a);
  yb = f(b);

  if ((ya < 0.0) == (yb < 0.0))
    {
      /* Fail because a and b do not bracket a root */
      *fail = 1;
      *nits = 0;
      return 0.0;
    }

  /* Try the midpoint of the interval [a,b] */
  midpoint = 0.5 * (a + b);
  while (a != midpoint && b != midpoint && n < 100)
    {
      ym = f(midpoint);
      if ((ya < 0.0) == (ym < 0.0))
        {
          /* The sign of f(a) is the same as the sign of f(midpoint),
             so move a to the midpoint */
          a = midpoint;
          ya = f(a);
        }
      else
        {
          /* The sign of f(b) is the same as the sign of f(midpoint),
             so move b to the midpoint */
          b = midpoint;
          yb = f(b);
        }
      midpoint = 0.5 * (a + b);
      n++;
      /* Each time round the loop the interval [a,b] gets smaller,
         until eventually a, b and midpoint converge */
    }

  if (n < 100)
    *fail = 0;
  else
    {
      /* Fail because hit the maximum number of iterations allowed */
      *fail = 2;
      midpoint = 0.0;
    }

  *nits = n;
  return midpoint;
}
